#!/usr/bin/pike7.4

/* Copyright (C) 2000-2004  Thomas Bopp, Thorsten Hampel, Ludger Merkens
 * Copyright (C) 2003-2004  Martin Baehr
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: debug.pike.in,v 1.1 2005/03/14 04:27:51 exodusd Exp $
 */

constant cvs_version="$Id: debug.pike.in,v 1.1 2005/03/14 04:27:51 exodusd Exp $";


inherit "../config";
inherit "applauncher.pike";

static object oInstall;

class Handler
{
  inherit Tools.Hilfe.Evaluator;

  void create(mapping _constants)
  {
    write=predef::write;
    ::create();
    constants+=_constants;
  }
}

object find_object(int id)
{
    if ( objectp(oInstall) )
        oInstall->find_object(id);
    return 0;
}

object handler, conn;
mapping vars=([]);

int main(int argc, array(string) argv)
{
  mapping options=init(argv);
  object _Server=conn->SteamObj(0);

  //write("%O\n", options);
  //Tools.Hilfe.StdinHilfe();
  handler = Handler(([
    "_Server"     : _Server,
    "conn"        : conn,
    "find_object" : conn->find_object,
    "modusers"    : _Server->get_module("users"),
    "modgroups"   : _Server->get_module("groups"),
    "edit"        : applaunch,
    "create"      : create_object,
    ]));

  array history=(Stdio.read_file(options->historyfile)||"")/"\n";
  if(history[-1]!="")
    history+=({""});

  Stdio.Readline.History readline_history=Stdio.Readline.History(512, history);

  readln->enable_history(readline_history);

  handler->add_input_line("start backend");

  string command;
  while(command=readln->read(
           sprintf("%s", (handler->state->finishedp()?"> ":">> "))))
  {
    if(sizeof(command))
    {
      Stdio.write_file(options->historyfile, readln->get_history()->encode());
      handler->add_input_line(command);
    }
  }
  handler->add_input_line("exit");
}

mapping init(array argv)
{
  mapping options = ([ "file":"/etc/shadow" ]);

  array opt=Getopt.find_all_options(argv,aggregate(
    ({"file",Getopt.HAS_ARG,({"-f","--file"})}),
    ({"host",Getopt.HAS_ARG,({"-h","--host"})}),
    ({"user",Getopt.HAS_ARG,({"-u","--user"})}),
    ));

  options->historyfile=getenv("HOME")+"/.steam_history";

  foreach(opt, array option)
  {
    options[option[0]]=option[1];
  }
  if(!options->host)
    options->host="localhost";
  if(!options->user)
    options->user="root";

  string server_path = "../";
  string config_path = "/etc/steam/";

  master()->add_include_path(server_path+"/server/include");
  master()->add_program_path(server_path+"/server/");
  master()->add_program_path(server_path+"/conf/");
  master()->add_program_path(server_path+"/server/net/coal/");
  add_constant("find_object", find_object);
  read_configs(config_path+"/config.txt");

  conn = ((program)"connection.pike")();
  string pw;
  readln->set_echo( 0 );
  pw=readln->read(sprintf("passwd for %s@%s: ", options->user, options->host));
  readln->set_echo( 1 );

  conn->start(options->host, (int)vars["port"], options->user, pw);

  return options;
}

// create new sTeam objects
// with code taken from the web script create.pike
mixed create_object(string objectclass, string name, void|string desc, void|mapping data)
{
  object _Server=conn->SteamObj(0);
  object created;
  object factory;

  if ( !stringp(objectclass))
    return "No object type submitted";

  factory = _Server->get_factory(objectclass);

  switch(objectclass)
  {
    case "exit":
      if(!data->exit_from)
        return "exit_from missing";
      break;
    case "link":
      if(!data->link_to)
        return "link_to missing";
      break;
  }

  if(!data)
    data=([]);
  created = factory->execute(([ "name":name ])+ data );

  if(stringp(desc))
    created->set_attribute("OBJ_DESC", desc);

//  if ( kind=="gallery" )
//  {
//    created->set_acquire_attribute("xsl:content", 0);
//    created->set_attribute("xsl:content",
//      ([ _STEAMUSER:_FILEPATH->path_to_object("/stylesheets/gallery.xsl") ])
//                          );
//  }

//  created->move(this_user());

  return created;
}

